@@ -1,26 +1,19 @@
 package AnyEvent::Filesys::Notify;
 
-# ABSTRACT: An AnyEvent compatible module to monitor files/directories for changes
-
-use Moo;
-use Moo::Role ();
-use MooX::late;
-use namespace::sweep;
+use Moose;
 use AnyEvent;
-use Path::Iterator::Rule;
+use File::Find::Rule;
 use Cwd qw/abs_path/;
 use AnyEvent::Filesys::Notify::Event;
 use Carp;
 use Try::Tiny;
 
-our $VERSION = '0.23_10';
-my $AEFN = 'AnyEvent::Filesys::Notify';
+our $VERSION = '0.02';
 
 has dirs        => ( is => 'ro', isa => 'ArrayRef[Str]', required => 1 );
 has cb          => ( is => 'rw', isa => 'CodeRef',       required => 1 );
 has interval    => ( is => 'ro', isa => 'Num',           default  => 2 );
 has no_external => ( is => 'ro', isa => 'Bool',          default  => 0 );
-has backend     => ( is => 'ro', isa => 'Str',           default  => '' );
 has filter      => ( is => 'rw', isa => 'RegexpRef|CodeRef' );
 has _fs_monitor => ( is => 'rw', );
 has _old_fs => ( is => 'rw', isa => 'HashRef' );
@@ -31,8 +24,25 @@ sub BUILD {
 
     $self->_old_fs( _scan_fs( $self->dirs ) );
 
-    $self->_load_backend;
-    return $self->_init;    # initialize the backend
+    if ( $self->no_external ) {
+        with 'AnyEvent::Filesys::Notify::Role::Fallback';
+    } elsif ( $^O eq 'linux' ) {
+        try { with 'AnyEvent::Filesys::Notify::Role::Linux' }
+        catch {
+            croak
+              "Unable to load the Linux plugin. You may want to install Linux::INotify2 or specify 'no_external' (but that is very inefficient):\n$_";
+        }
+    } elsif ( $^O eq 'darwin' ) {
+        try { with 'AnyEvent::Filesys::Notify::Role::Mac' }
+        catch {
+            croak
+              "Unable to load the Mac plugin. You may want to install Mac::FSEvents or specify 'no_external' (but that is very inefficient):\n$_";
+        }
+    } else {
+        with 'AnyEvent::Filesys::Notify::Role::Fallback';
+    }
+
+    return $self->_init;
 }
 
 sub _process_events {
@@ -73,17 +83,11 @@ sub _scan_fs {
     # Accept either an array of dirs or a array ref of dirs
     my @paths = ref $args[0] eq 'ARRAY' ? @{ $args[0] } : @args;
 
-    my $fs_stats = {};
+    # Separated into two lines to avoid stat on files multiple times.
+    my %files = map { $_ => 1 } File::Find::Rule->in(@paths);
+    %files = map { abs_path($_) => _stat($_) } keys %files;
 
-    my $rule = Path::Iterator::Rule->new;
-    my $next = $rule->iter(@paths);
-    while ( my $file = $next->() ) {
-        my $stat = _stat($file)
-          or next; # Skip files that we can't stat (ie, broken symlinks on ext4)
-        $fs_stats->{ abs_path($file) } = $stat;
-    }
-
-    return $fs_stats;
+    return \%files;
 }
 
 sub _diff_fs {
@@ -125,106 +129,38 @@ sub _diff_fs {
 sub _is_path_modified {
     my ( $old_path, $new_path ) = @_;
 
-    return 1 if $new_path->{mode} != $old_path->{mode};
     return   if $new_path->{is_dir};
     return 1 if $new_path->{mtime} != $old_path->{mtime};
     return 1 if $new_path->{size} != $old_path->{size};
     return;
 }
 
-# Originally taken from Filesys::Notify::Simple --Thanks Miyagawa
+# Taken from Filesys::Notify::Simple --Thanks Miyagawa
 sub _stat {
     my $path = shift;
 
     my @stat = stat $path;
-
-    # Return undefined if no stats can be retrieved, as it happens with broken
-    # symlinks (at least under ext4).
-    return undef unless @stat;
-
     return {
         path   => $path,
         mtime  => $stat[9],
         size   => $stat[7],
-        mode   => $stat[2],
         is_dir => -d _,
     };
 
 }
 
-# Figure out which backend to use:
-# I would prefer this to be done at compile time not object build, but I also
-# want the user to be able to force the Fallback role. Something like an
-# import flag would be great, but Moose creates an import sub for us and
-# I'm not sure how to cleanly do it. Maybe need to use traits, but the
-# documentation suggests traits are for application of roles by object.
-# This will work for now.
-sub _load_backend {
-    my $self = shift;
-
-    if ( $self->backend ) {
-
-        # Use the AEFN::Role prefix unless the backend starts with a +
-        my $prefix = "${AEFN}::Role::";
-        my $backend = $self->backend;
-        $backend = $prefix . $backend unless $backend =~ s{^\+}{};
-
-        try { Moo::Role->apply_roles_to_object( $self, $backend ); }
-        catch {
-            croak "Unable to load the specified backend ($backend). You may "
-              . "need to install Linux::INotify2, Mac::FSEvents or IO::KQueue:"
-              . "\n$_";
-        }
-    } elsif ( $self->no_external ) {
-        Moo::Role->apply_roles_to_object( $self, "${AEFN}::Role::Fallback" );
-    } elsif ( $^O eq 'linux' ) {
-        try { Moo::Role->apply_roles_to_object( $self, "${AEFN}::Role::Inotify2"); }
-        catch {
-            croak "Unable to load the Linux plugin. You may want to install "
-              . "Linux::INotify2 or specify 'no_external' (but that is very "
-              . "inefficient):\n$_";
-        }
-    } elsif ( $^O eq 'darwin' ) {
-        try { Moo::Role->apply_roles_to_object( $self, "${AEFN}::Role::FSEvents" ); }
-        catch {
-            croak "Unable to load the Mac plugin. You may want to install "
-              . "Mac::FSEvents or specify 'no_external' (but that is very "
-              . "inefficient):\n$_";
-        }
-    } elsif ( $^O eq 'freebsd' ) {
-        try { Moo::Role->apply_roles_to_object( $self, "${AEFN}::Role::KQueue" ); }
-        catch {
-            croak "Unable to load the FreeBSD plugin. You may want to install "
-              . "IO::KQueue or specify 'no_external' (but that is very "
-              . "inefficient):\n$_";
-        }
-    } else {
-        Moo::Role->apply_roles_to_object( $self, "${AEFN}::Role::Fallback" );
-    }
-
-    return 1;
-}
-
 1;
 
-__END__
-
-=pod
-
 =head1 NAME
 
 AnyEvent::Filesys::Notify - An AnyEvent compatible module to monitor files/directories for changes
 
-=head1 VERSION
-
-version 0.23_10
-
 =head1 SYNOPSIS
 
     use AnyEvent::Filesys::Notify;
 
     my $notifier = AnyEvent::Filesys::Notify->new(
-        dirs     => [ qw( this_dir that_dir ) ],
+        dir      => [ qw( this_dir that_dir ) ],
         interval => 2.0,             # Optional depending on underlying watcher
         filter   => sub { shift !~ /\.(swp|tmp)$/ },
         cb       => sub {
@@ -273,9 +209,9 @@ Specifies the time in fractional seconds between file system checks for
 the L<AnyEvent::Filesys::Notify::Role::Fallback> implementation.
 
 Specifies the latency for L<Mac::FSEvents> for the
-C<AnyEvent::Filesys::Notify::Role::FSEvents> implementation.
+C<AnyEvent::Filesys::Notify::Role::Mac> implementation.
 
-Ignored for the C<AnyEvent::Filesys::Notify::Role::Inotify2> implementation.
+Ignored for the C<AnyEvent::Filesys::Notify::Role::Linux> implementation.
 
 =item filter
 
@@ -295,49 +231,28 @@ A CodeRef that is called when a modification to the monitored directory(ies) is
 detected. The callback is passed a list of
 L<AnyEvent::Filesys::Notify::Event>s. Required.
 
-=item backend
-
-    backend => 'Fallback',
-    backend => 'FreeBSD',
-    backend => '+My::Filesys::Notify::Role::Backend',
-
-Force the use of the specified backend. The backend is assumed to have the
-C<AnyEvent::Filesys::Notify::Role> prefix, but you can force a fully qualified
-name by prefixing it with a plus. Optional.
-
 =item no_external
 
     no_external => 1,
 
-This is retained for backward compatibility. Using C<backend => 'Fallback'>
-is preferred. Force the use of the L</Fallback> watcher implementation. This is
-not encouraged as the L</Fallback> implement is very inefficient, but it does
-not require either L<Linux::INotify2> nor L<Mac::FSEvents>. Optional.
+Force the use of the L</Fallback> watcher implementation. This is not
+encouraged as the L</Fallback> implement is very inefficient, but it 
+does not require either L<Linux::INotify2> nor L<Mac::FSEvents>. Optional.
 
 =back
 
 =head1 WATCHER IMPLEMENTATIONS
 
-=head2 INotify2 (Linux)
+=head2 Linux
 
 Uses L<Linux::INotify2> to monitor directories. Sets up an C<AnyEvent-E<gt>io>
 watcher to monitor the C<$inotify-E<gt>fileno> filehandle.
 
-=head2 FSEvents (Mac)
+=head2 Mac
 
 Uses L<Mac::FSEvents> to monitor directories. Sets up an C<AnyEvent-E<gt>io>
 watcher to monitor the C<$fsevent-E<gt>watch> filehandle.
 
-=head2 KQueue (FreeBSD/Mac)
-
-Uses L<IO::KQueue> to monitor directories. Sets up an C<AnyEvent-E<gt>io>
-watcher to monitor the C<IO::KQueue> object.
-
-B<WARNING> - L<IO::KQueue> and the C<kqueue()> system call require an open
-filehandle for every directory and file that is being watched. This makes
-it impossible to watch large directory structures (and inefficient to watch
-moderately sized directories). The use of the KQueue backend is discouraged.
-
 =head2 Fallback
 
 A simple scan of the watched directories at regular intervals. Sets up an
@@ -372,6 +287,7 @@ are perfect for many situations. If one of their modules will work for you
 by all means use it, but if you are already using an event loop, this
 module may fit the bill.
 
+
 =head1 SEE ALSO
 
 Modules used to implement this module L<AnyEvent>, L<Mac::FSEvents>,
@@ -383,25 +299,16 @@ Alternatives to this module L<Filesys::Notify::Simple>, L<File::ChangeNotify>.
 
 Please report any bugs or suggestions at L<http://rt.cpan.org/>
 
-Forcing the C<IO::KQueue> backend on a Mac does not seem to work.  The
-C<IO::KQueue> backend seems to be working fine on FreeBSD. I don't have the
-experience or time to fix it on a Mac.  I would greatly appreciate any help
-troubleshooting this.
-
-=head1 CONTRIBUTORS
-
-Thanks to Gasol Wu E<lt>gasol.wu@gmail.comE<gt> who contributed the FreeBSD
-support for IO::KQueue.
-
 =head1 AUTHOR
 
 Mark Grimes, E<lt>mgrimes@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by Mark Grimes, E<lt>mgrimes@cpan.orgE<gt>.
+Copyright (C) 2009 by Mark Grimes
 
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either Perl version 5.8.2 or,
+at your option, any later version of Perl 5 you may have available.
 
 =cut
